#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from abc import ABC

class BaseUptimeService(ABC):

  def __init__(self, api_token, filter_opts):
    pass
  
  # the implementation of this function must return a data structure as follows
  # (to be later passed to check_get_all_summary_avg as parameter):
  # {
  #   "count": VALUE, # count of checks retrieved
  #   "_data": VALUE  # coult be a list or a dict; implementation is service-dependent; not used outside class
  # }
  def check_get_all(self):
    pass

  # the implementation of this function must return a data structure as follows:
  # {
  #   "_meta":{
  #     "service_name": "SERVICE_NAME",
  #     "from_ts":      VALUE,
  #     "to_ts":        VALUE
  #   },
  #   "_checks":[
  #     {
  #       "name":  "CHECK_NAME",
  #       "id":    CHECK_ID,
  #       "tags":[
  #          "ONETAG",
  #          "ANOTHERTAG",
  #          "MORETAGS"
  #       ],
  #       "uptime":{
  #         "totaldown":    VALUE,
  #         "totalunknown": VALUE,
  #         "totalup":      VALUE,
  #         "pct":          VALUE
  #       }
  #     },
  #     {
  #       ...
  #     }
  #   ]
  # }
  def check_get_all_summary_avg(self, checks, from_ts, to_ts, report_progress_func=None):
    pass



#!/usr/bin/env python
#
# a simple class to abtract the Python 3.1 API
# https://docs.pingdom.com/api/
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import requests
import json
import logging
from .Base import BaseUptimeService


class Pingdom(BaseUptimeService):

  SERVICE_NAME='pingdom'
  PINGDOM_API_URL='https://api.pingdom.com/api/3.1'

  def __init__(self, api_token, filter_opts):
    self.api_token = api_token
    self.req_headers = {'Authorization': f'Bearer {api_token}'}
    self.filter_opts = filter_opts
    # try to open a connection to confirm correct token
    logging.debug('Attempt a checks API request to Pingdom to confirm token validity')
    self.__req_get('/checks?limit=1')

  def check_get_all(self):
    # we always want to retrieve the tags information in our request
    req_parameters = 'include_tags=true'
    # tags_include filter can be used here to optimize initial results, but only if our special 'none' tag is not present
    if self.filter_opts['tags_include'] and 'none' not in self.filter_opts['tags_include']:
      req_parameters += '&tags=' + ','.join(self.filter_opts['tags_include'])
    # make request
    req = self.__req_get(f'/checks?{req_parameters}')
    res = json.loads(req.text)
    # return structure
    checks = { 'count': len(res['checks']), '_data': res }
    return checks

  def check_get_all_summary_avg(self, checks, from_ts, to_ts, report_progress_func=None):
    checks_uptime = { '_meta': { 'service_name': self.SERVICE_NAME, 'from_ts': from_ts, 'to_ts': to_ts }, '_checks': [] }
    pr_total, pr_counter, pr_thr = checks['count'], 0, 10 # report every 10% increase
    for check in checks['_data']['checks']:
      pr_counter += 1
      check_tags = [tag['name'] for tag in check['tags']]
      logging.debug(check)
      # check filter options
      if self.__is_check_filtered(check, check_tags):
        logging.debug('Check {} skipped based on include/exclude filters'.format(check['id']))
        continue # skip check
      # process check
      summary = self.__check_get_summary_avg(check['id'], from_ts, to_ts)
      logging.debug(summary)
      status = summary['summary']['status']
      # additional info: calculate the percentage of uptime
      try:
        status['pct'] = status['totalup'] / (status['totalup'] + status['totaldown']) * 100
      except ZeroDivisionError:
        status['pct'] = -1 # check does not have up or down time, it must be all unknown
      # for each check, store uptime and tags
      checks_uptime['_checks'].append({
        'name': check['name'], 'id': check['id'],
        'tags': check_tags,
        'uptime': status
      })
      pr = round((pr_counter / pr_total * 100), 2)
      if report_progress_func and pr > pr_thr:
        pr_thr += 10
        report_progress_func(pr)
    return checks_uptime

  def __check_get_summary_avg(self, check_id, from_ts, to_ts):
    req = self.__req_get(f'/summary.average/{check_id}?from={from_ts}&to={to_ts}&includeuptime=true')
    return json.loads(req.text)

  # utility method to handle requests, will raise exception if request fails
  def __req_get(self, url):
    req = requests.get(self.PINGDOM_API_URL+url, headers=self.req_headers)
    req.raise_for_status()
    return req

  # utility method to process check/tag filters; returns true if check must be filtered
  def __is_check_filtered(self, check, check_tags):
    # check filter options
    if self.filter_opts['checks_exclude_paused'] and check['status'] == 'paused':
      return True # skip paused checks
    if self.filter_opts['checks_include'] and str(check['id']) not in self.filter_opts['checks_include']:
      return True # skip check not in include list
    if self.filter_opts['checks_exclude'] and str(check['id']) in self.filter_opts['checks_exclude']:
      return True # skip check in exclude list
    # tag filters options
    if self.filter_opts['tags_include']:
      if not any(tag in check_tags for tag in self.filter_opts['tags_include']):
        if 'none' not in self.filter_opts['tags_include']:
          return True # skip checks with tags not in include list
        elif check_tags:
          return True # skip checks which have tags, but none specified in the include list ('none' case)
    if self.filter_opts['tags_exclude']:
      if any(tag in check_tags for tag in self.filter_opts['tags_exclude']):
        return True # skip tags in exclude list
      elif 'none' in self.filter_opts['tags_exclude'] and not check_tags:
        return True # skip checks without tags, as 'none' is in exclude list
    # no filter applies to this check
    return False
 



#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .UptimeServices import *

def getService(service_name, api_token, filter_opts):
  service = [service for service in Base.BaseUptimeService.__subclasses__() if service_name == service.SERVICE_NAME]
  if service:
    return service[0](api_token, filter_opts)
  else:
    raise Exception(f'Uptime Service {service_name} not supported')

def getSupportedServices():
  return [service.SERVICE_NAME for service in Base.BaseUptimeService.__subclasses__()]




#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.


# if tags grouping is enabled, build structures based on tag hierarchy
# each level of hierarchy is separated by pipe and defined as a list of tags for example
# cats,dogs|apples,pears,trees|...
# this results in checks being classified based on the tags they have:
# --cats
#    \--apples
#    \--pears
#    \--trees
# --dogs
#    \--apples
#    \--pears
#    \--trees
# each tag has a "_checks" key containing a list of corresponding checks

import logging

def get_uptime_tags_groups(checks_uptime, tag_grouping):

  checks_uptime_tg = {'_meta': checks_uptime['_meta'], '_checks':[]}
  tags = [tag_group.split(',') for tag_group in tag_grouping.split('|')]
  logging.debug('Tag grouping enabled with hierarchy: {}'.format(tags))

  # a recursive function to populate the tag structure
  def place_in_tag_hierarchy(check_dict, check, tags, tag_level):
    added = False
    for tag in tags[tag_level]:
      # initialize tag space in the structure if not done
      if tag not in check_dict:
        check_dict[tag] = {'_checks':[]}
      # does the check have this tag?
      if tag in check['tags']:
        # found, check deeper level
        if tag_level+1 >= len(tags) or not place_in_tag_hierarchy(check_dict[tag], check, tags, tag_level+1):
          # if not found at deeper level, add at this level
          check_dict[tag]['_checks'].append(check)
        added = True # either at deeper level or at this one, this is now added
    # if at level 0 and not found anywhere, add at outer level
    if tag_level == 0 and not added:
      check_dict['_checks'].append(check)
      added = True
    return added

  # populate tag structure
  for check in checks_uptime['_checks']:
    place_in_tag_hierarchy(checks_uptime_tg, check, tags, 0)

  return checks_uptime_tg



#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging


# aggregate statistics, to produce more complete reports;
# -1 values (i.e. checks fully paused for measured interval) are excluded
# no return value, the dict is transformed inline
def add_aggregated_stats(checks_data):
  # for each level process tags first, and _checks at that level last
  for check_index in sorted(checks_data.keys(),reverse=True):
    if check_index[0] != '_':
      add_aggregated_stats(checks_data[check_index])
    elif check_index == '_checks':
      checks_data['_stats'] = { 'g_checks_count': 0, 'g_pct_avg': 0, 'gs_checks_count': 0, 'gs_pct_avg': 0 }
      checks_data['_stats']['g_checks_count'] = len([check for check in checks_data[check_index] if check['uptime']['pct'] > -1])
      checks_data['_stats']['g_pct_avg'] = sum(
        [check['uptime']['pct'] for check in checks_data[check_index] if check['uptime']['pct'] > -1 ]
        ) / checks_data['_stats']['g_checks_count'] if checks_data['_stats']['g_checks_count'] != 0 else -1
      checks_data['_stats']['gs_checks_count'] = sum(
        [checks_data[tag]['_stats']['gs_checks_count'] for tag in sorted(checks_data.keys(),reverse=True) if tag[0] != '_']
        ) + checks_data['_stats']['g_checks_count']
      # for the group + subgroup percentage uptime, use a weighted average
      count_and_pct_for_subs = [[checks_data[tag]['_stats']['gs_pct_avg'], checks_data[tag]['_stats']['gs_checks_count']]
        for tag in sorted(checks_data.keys(),reverse=True) if tag[0] != '_']
      count_and_pct_for_subs.append([checks_data['_stats']['g_pct_avg'], checks_data['_stats']['g_checks_count']])
      checks_data['_stats']['gs_pct_avg'] = weighted_avg(count_and_pct_for_subs)
	

# utility function to calculate weighted average
# input is [[val1, w1], [val2, w2], ...]
def weighted_avg(val_and_w):
  # calculate sum of weights
  wsum = sum([w for val,w in val_and_w])
  # apply each weight as decimal percentage to each value and sum them
  wavg = sum([(val * (w/wsum)) for val,w in val_and_w]) if wsum != 0 else -1
  return wavg



#!/usr/bin/env python
#
# uptime-reporting: a tool to report on service availability statistics
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import sys, os, re
import argparse
import logging
from datetime import datetime
import UptimeReporting.UptimeService
import UptimeReporting.TagsGrouping
import UptimeReporting.DateRange
import UptimeReporting.Report
import UptimeReporting.Aggreg


def main():

  # validation functions
  def arg_vld_report_thresholds(s):
    if not re.match('^\d+\.?\d*,\d+\.?\d*$',s):
      raise argparse.ArgumentTypeError('Wrong format for report threshold parameter')
    return s
  def arg_vld_date(s):
    if not re.match('(^last$)|(^[0-9]{4}-[0-9]{2}-[0-9]{2}$)|(^[0-9]{4}-[0-9]{2}-[0-9]{2})_([0-9]{4}-[0-9]{2}-[0-9]{2}$)',s):
      raise argparse.ArgumentTypeError('Wrong format for date parameter; use "last" or YYYY-MM-DD or YYYY-MM-DD_YYYY-MM-DD depending on report type')
    return s
  def arg_vld_list_str(s):
    return s.split(',') if s else []

  # get supported uptime services
  supported_services = UptimeReporting.UptimeService.getSupportedServices()
  # parse input arguments
  args_required = ('--version' not in sys.argv)
  parser_example_text = '''example:\n\n  uptime-reporting.py -t API_TOKEN --report weekly --date last\n '''
  parser = argparse.ArgumentParser(description='Report statistics on uptime checks', epilog=parser_example_text, formatter_class=argparse.RawDescriptionHelpFormatter)
  parser.add_argument('--service', help='Uptime Service for data collection; supported services: {}'.format(','.join(supported_services)), dest='service', action='store', choices=supported_services, required=args_required)
  parser.add_argument('--token', help='Uptime Service API token for authentication', dest='api_token', action='store', required=args_required)
  parser.add_argument('--date', help='Date in YYYY-MM-DD (used to identify month or week for weekly/monthly report type); use "last" as convenient way to get report for last week or last month; if using range report type, then use format "YYYY-MM-DD_YYYY-MM-DD")', action='store', dest='date', required=args_required, type=arg_vld_date)
  parser.add_argument('--report-type', help='Report type', action='store', dest='report_type', required=args_required, choices=['weekly','monthly','range'])
  parser.add_argument('--report-format', help='Report format (default: text); text is a terminal-friendly report, jinja uses templates to produce custom format reports', action='store', dest='report_format', choices=['text','jinja'], default='text')
  parser.add_argument('--report-thresholds', help='Set thresholds for warning and critical uptime status (default "99.95,99.00"). These are typically used to color-code uptime stats in the reports.', action='store', dest='report_thresholds', default='99.95,99.00', type=arg_vld_report_thresholds)
  parser.add_argument('--report-text-colors', help='Shows threshold colors in terminal-friendly text report', action='store_true', dest='report_text_colors', default=False)
  parser.add_argument('--report-jinja-template', help='Report jinja template for custom format (default: default.html). If a path is not specified, uses the templates available under templates/ module dir.', action='store', dest='report_jinja_template', default='default.html')
  parser.add_argument('--report-jinja-flags', help='A comma-separated list of flags that can be used within the jinja template to customize generation of the report', action='store', type=arg_vld_list_str, dest='report_jinja_flags', default=[])
  parser.add_argument('--report-filename', help='If specified, save report to file instead of printing on screen', action='store', dest='report_filename', default=None)
  parser.add_argument('--tags-grouping', help='If specified, groups checks and aggregates statistics based on tags; different hierarchy levels can be specified; see documentation for details', action='store', dest='tags_grouping', default=None)
  parser.add_argument('--tags-exclude', help='A comma-separated list of tags; checks associated with these tags will be excluded from the report. Use "none" to exclude checks without any tag.', action='store', type=arg_vld_list_str, dest='tags_exclude', default=None)
  parser.add_argument('--tags-include', help='A comma-separated list of tags; if specified, only checks associated with at least one of these tags will be included in the report. Use "none" to include checks without any tag.', action='store', type=arg_vld_list_str, dest='tags_include', default=None)
  parser.add_argument('--checks-exclude', help='A comma-separated list of checks Id, to be excluded from the report.', action='store', type=arg_vld_list_str, dest='checks_exclude', default=None)
  parser.add_argument('--checks-exclude-paused', help='Skip paused check in the report', action='store_true', dest='checks_exclude_paused', default=False)
  parser.add_argument('--checks-include', help='A comma-separated list of checks Id; if specified, only these checks will be included in the report.', action='store', type=arg_vld_list_str, dest='checks_include', default=None)
  parser.add_argument('--verbosity', help='Verbosity of output: 0 = quiet, 1 = standard (default), 2 = debug', action='store', dest='verbosity', choices=['0','1','2'], default=1)
  parser.add_argument('--version', help='Display version and exit', action='store_true', dest='version')
  args = parser.parse_args()

  if args.version:
    print(UptimeReporting.__version__)
    sys.exit(0)

  # logging
  root = logging.getLogger()
  root.setLevel(logging.DEBUG if args.verbosity == '2' else logging.INFO if args.verbosity == '1' else logging.ERROR)
  printver = print if args.verbosity != '0' else lambda *a, **k: None

  # validate and further parse date input
  printver(f'Report type: {args.report_type}')
  date_range = UptimeReporting.DateRange.get_date_range(args.report_type, args.date)
  printver('Date range: {} - {}'.format(date_range['start'],date_range['end']))

  # get all the checks, along with tags
  service_filter_options = {
    'checks_exclude_paused': args.checks_exclude_paused,
    'checks_include': args.checks_include, 'checks_exclude': args.checks_exclude,
    'tags_include': args.tags_include, 'tags_exclude': args.tags_exclude
  }
  service = UptimeReporting.UptimeService.getService(service_name=args.service, api_token=args.api_token, filter_opts=service_filter_options)
  logging.debug('Test connection to uptime service successful, retrieving all checks')
  checks = service.check_get_all()

  # for each check, retrieve the uptime stats for the desired range
  printver('Found {} checks to evaluate, gathering uptime stats for each'.format(checks['count']))
  def print_progress(progress): # utility function for progress report
    if args.verbosity != '0':
      sys.stdout.write("Progress: {}%   \r".format(progress))
      sys.stdout.flush()
  checks_uptime = service.check_get_all_summary_avg(checks, date_range['start_ts'], date_range['end_ts'], print_progress)

  # if tag grouping is enabled, build a data structure based on tag hierarchy
  if args.tags_grouping:
    printver('Grouping checks based on tag hierarchy: {}'.format(args.tags_grouping))
    checks_uptime = UptimeReporting.TagsGrouping.get_uptime_tags_groups(checks_uptime, args.tags_grouping)

  # add aggregated statistics
  printver('Aggregating check uptime statistics')
  UptimeReporting.Aggreg.add_aggregated_stats(checks_uptime)

  # report results
  printver('Writing report')
  report = UptimeReporting.Report.Report(
    checks_uptime, checks_date_range=date_range, report_type=args.report_format, thresholds=args.report_thresholds,
    jinja_template=args.report_jinja_template, jinja_flags=args.report_jinja_flags, text_use_colors=args.report_text_colors
  )
  rendered_report = report.render()
  if args.report_filename:
    with open(args.report_filename, 'w') as f:
      f.write(rendered_report)
    printver(f'Report written to {args.report_filename}')
  else:
    print(rendered_report)


if __name__=="__main__":
  main()



#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, date, timedelta
import calendar
import logging
import re

# validate and get range information
def get_date_range(p_report_type, p_date):
  date_range = {}
  logging.debug(f'Date parameter: {p_date}')
  # range report
  if p_report_type == 'range':
    date_range_m = re.match(r'([0-9]{4}-[0-9]{2}-[0-9]{2})_([0-9]{4}-[0-9]{2}-[0-9]{2})$', p_date)
    if not date_range_m:
      raise Exception('ERROR: date is not in YYYY-MM-DD_YYYY-MM-DD format')
    logging.debug(f'Report type {p_report_type}, analysing range {p_date}')
    date_range = {
      'start': datetime.strptime(date_range_m.group(1), '%Y-%m-%d'),
      'end': datetime.strptime(date_range_m.group(2), '%Y-%m-%d')
    }
    if date_range['start'] >= date_range['end']:
      raise Exception('ERROR: in date range, start date should be older than end date')
  # weekly report
  if p_report_type == 'weekly':
    date_range_m = re.match(r'([0-9]{4}-[0-9]{2}-[0-9]{2})$|(last)$', p_date)
    if not date_range_m:
      raise Exception('ERROR: date is not in YYYY-MM-DD format or \'last\'')
    if p_date == 'last':
      ref_date = datetime.today() + timedelta(weeks=-1)
    else:
      ref_date = datetime.strptime(p_date, '%Y-%m-%d')
    logging.debug(f'Report type {p_report_type}, identifying weekly date range around {ref_date}')
    start_date = ref_date + timedelta(-ref_date.weekday())
    end_date = start_date + timedelta(days=6)
    date_range = { 'start': start_date, 'end': end_date }
  # monthly report
  if p_report_type == 'monthly':
    date_range_m = re.match(r'([0-9]{4}-[0-9]{2}-[0-9]{2})$|(last)$', p_date)
    if not date_range_m:
      raise Exception('ERROR: date is not in YYYY-MM-DD format or \'last\'')
    if p_date == 'last':
      ref_date = datetime.today().replace(day=1) + timedelta(days=-1)
    else:
      ref_date = datetime.strptime(p_date, '%Y-%m-%d')
    logging.debug(f'Report type {p_report_type}, identifying monthly date range around {ref_date}')
    month_c = calendar.monthrange(ref_date.year, ref_date.month)
    start_date = ref_date.replace(day=1)
    end_date = ref_date.replace(day=month_c[1])
    date_range = { 'start': start_date, 'end': end_date }
  # for all report types, ensure that start date has time 00:00
  # and end date has time 23:59:59 and also store a timestamp version
  date_range['start']= date_range['start'].replace(hour=0,minute=0,second=0,microsecond=0)
  date_range['end']= date_range['end'].replace(hour=23,minute=59,second=59,microsecond=0)
  date_range['start_ts'] = int(date_range['start'].timestamp())
  date_range['end_ts'] = int(date_range['end'].timestamp())
  return date_range



from setuptools import setup, find_packages
import codecs
from glob import glob
import os.path

def read(rel_path):
  here = os.path.abspath(os.path.dirname(__file__))
  with codecs.open(os.path.join(here, rel_path), 'r') as fp:
    return fp.read()

def get_version(rel_path):
  for line in read(rel_path).splitlines():
    if line.startswith('__version__'):
      delim = '"' if '"' in line else "'"
      return line.split(delim)[1]
    else:
      raise RuntimeError("Unable to find version string.")

setup(
  name="uptime-reporting",
  version=get_version("UptimeReporting/__init__.py"),
  packages=find_packages(),
  python_requires='~=3.7',

  entry_points={
    "console_scripts": [
      "uptime-reporting = UptimeReporting.__main__:main"
    ]
  },

  package_data={
    "UptimeReporting": ["Templates/*"]
  },

  data_files=[
    ('docs', glob('docs/**')),
    ('.', ['./LICENSE'])
  ],

  install_requires=[
    "Jinja2>=2.11.1",
    "requests>=2.23.0",
    "tabulate>=0.8.7"
  ],

  # metadata to display on PyPI
  author="Giacomo Lozito",
  author_email="giacomo.lozito@gmail.com",
  description="service uptime reporting tool",
  long_description=read('README.md'),
  long_description_content_type='text/markdown',
  license='GPLv3',
  keywords="service uptime reporting pingdom",
  url="https://github.com/giacomolozito/uptime-reporting",
  classifiers=[
    "Development Status :: 4 - Beta",
    "Environment :: Console",
    "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
    "Programming Language :: Python :: 3.7"
  ]
)



#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.	See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

import logging
import os
from tabulate import tabulate
import jinja2
from datetime import datetime


class Report:

  def __init__(self, checks_data, checks_date_range, report_type, thresholds, jinja_template='default', jinja_flags=[], text_use_colors=False):
    self.render = self.render_jinja if report_type == 'jinja' else self.render_text
    self.jinja_template = jinja_template
    self.jinja_flags = jinja_flags
    self.thresholds = dict(zip(['w','c'],[float(x) for x in thresholds.split(',')]))
    self.checks_data = checks_data
    self.checks_date_range = checks_date_range
    self.text_use_colors = text_use_colors

  def render_text(self):
    # a terminal-friendly report leveraging tabulate module for pretty-printing
    headers=['Check Name','Uptime %', 'Downtime']
    def render_text_iter(checks_data, label=[], minsize=80):
      # print aggregated stats for this level
      report = ''
      group_name = ' + '.join(label).upper() if label else 'ALL CHECKS'
      stats_row = [self.__color_text([f'AVERAGE UPTIME {group_name}', round(checks_data['_stats']['gs_pct_avg'],4), ''], checks_data['_stats']['gs_pct_avg'])]
      # print checks uptime results for this level
      checks_rows = [self.__color_text([row['name'], round(row['uptime']['pct'],4), self.__seconds_to_dhms(row['uptime']['totaldown'])], row['uptime']['pct'])
        for row in checks_data['_checks']]
      sizing_row = [['-' * minsize, '', '']] # sizing row, used to equally-size all tables
      # put together and print
      report += ('|  ' * len(label)) + '\n'
      report += ('|  ' * len(label)) + f'\-- Uptime statistics {group_name}\n'
      for report_line in tabulate(stats_row + checks_rows + sizing_row, headers=headers, numalign='right').splitlines():
        report += ('|  ' * (len(label)+1)) + report_line + '\n'
      for tag in [tag for tag in checks_data.keys() if tag[0] != '_']:
        report += render_text_iter(checks_data[tag], label=(label + [tag]))
      return report
    report = render_text_iter(self.checks_data)
    return report

  def render_jinja(self):
    # if specified without path, pick template from the module templates dir; otherwise assume it's user-provided
    if os.path.basename(self.jinja_template) == self.jinja_template:
      logging.debug(f'Using template {self.jinja_template} from module templates dir')
      template_loader=jinja2.PackageLoader('UptimeReporting', 'Templates')
    else:
      logging.debug(f'Using template {self.jinja_template}, user-provided')
      template_searchpath = os.path.dirname(self.jinja_template)
      template_loader = jinja2.FileSystemLoader(searchpath=template_searchpath)
    template_env = jinja2.Environment(loader=template_loader)
    # custom filters:
    # - a timedelta filter to make duration in seconds more readable
    # - a filter to return threshold status 0,1,2 (ok, warning, critical) based on % value vs thresholds
    template_env.filters['seconds_to_dhms'] = self.__seconds_to_dhms
    template_env.filters['threshold_status'] = self.__threshold_status
    # render
    template = template_env.get_template(os.path.basename(self.jinja_template))
    report = template.render(
      checks_data=self.checks_data, checks_thresholds=self.thresholds,
      checks_date_range=self.checks_date_range, flags=self.jinja_flags, now=datetime.now()
    )
    return report

  # utility function to color text based on thresholds - used in the text report if colors are enabled
  def __color_text(self, strings, value):
    if self.text_use_colors and value > -1:
      color = ['\033[0;32m', '\033[0;33m', '\033[0;31m', '\033[0;37m'][self.__threshold_status(value)]
      return [f'{color}{string}\033[0;37m' for string in strings]
    return strings # no color

  # utility function to return uptime check status against threshold; 0 = ok, 1 = warning, 2 = critical, 3 = na
  def __threshold_status(self, uptime_pct):
    if uptime_pct < 0:
      return 3
    elif uptime_pct > self.thresholds['w']:
      return 0
    elif uptime_pct <= self.thresholds['w'] and uptime_pct > self.thresholds['c']:
      return 1
    else:
      return 2

  # utility function to convert seconds in a human-readable "?d ?h ?m ?s" string (days, hours, minutes, seconds)
  def __seconds_to_dhms(self, seconds):
    d, rem = divmod(seconds, 86400)
    h, rem = divmod(rem, 3600)
    m, s = divmod(rem, 60)
    locals_ = locals()
    dhms = ' '.join(["{n}{mag}".format(n=int(locals_[mag]), mag=mag) for mag in ['d','h','m','s'] if locals_[mag]])
    return dhms
